from .DAQmxTypes import TaskHandle
from . import DAQmxFunctions
from .DAQmxFunctions import *
import ctypes


# Create a list of the name of the function that uses TastHandle as the first argument
# All the function of this list will be converted to method of the task object
# The name of the method will be the same name as the name of the DAQmx function without the 
# the DAQmx in front of the name
task_function_list = [name for name in function_dict.keys() if \
                 len(function_dict[name]['arg_type'])>0 and \
                 (function_dict[name]['arg_type'][0] is TaskHandle) and\
                 'task' in function_dict[name]['arg_name'][0]]

# Remove ClearTask in task_functon_list
task_function_list = [name for name in task_function_list if name not in ['DAQmxClearTask']]

try :
    from .DAQmxCallBack import *
    _callback = True
except NotImplementedError:
    _callback = False

if _callback:
    class CallbackParent(object):
        _EveryNSamplesEvent_already_register = False
        def AutoRegisterEveryNSamplesEvent(self, everyNsamplesEventType,nSamples,options, name='EveryNCallback'):
            """Register the method named name as the callback function for EveryNSamplesEvent
            
            
            With this method you can register a method of the class Task as a callback function. 
            The parameters everyNsamplesEventType, nSamples and options are the same 
            as the DAQmxRegisterEveryNSamplesEvent parameters
            
            No parameters are passed to the method  

            If an event was already registered, the UnregisterEveryNSamplesEvent is automatically called      
            """
            if self._EveryNSamplesEvent_already_register:
                self.UnregisterEveryNSamplesEvent()
            self_id = create_callbackdata_id(self)
            # Define the python function
            def EveryNCallback_py(taskHandle, everyNsamplesEventType, nSamples, self_id):
                self = get_callbackdata_from_id(self_id)
                getattr(self,name)()
                return 0
            # Transform the python function to a CFunction
            self.EveryNCallback_C = DAQmxEveryNSamplesEventCallbackPtr(EveryNCallback_py)
            # Register the function
            self.RegisterEveryNSamplesEvent(everyNsamplesEventType,nSamples,options,self.EveryNCallback_C,self_id)
            self._EveryNSamplesEvent_already_register = True
        def UnregisterEveryNSamplesEvent(self):
            self.RegisterEveryNSamplesEvent(1,0,0,ctypes.cast(None, DAQmxEveryNSamplesEventCallbackPtr),0)
            self._EveryNSamplesEvent_already_register = False

        def AutoRegisterDoneEvent(self, options, name='DoneCallback'):
            """Register the method named name as the callback function for DoneEvent
            
            With this method you can register a method of the class Task as a callback function. 
            The parameter options is the same as the DAQmxRegisterDoneEvent parameters
            
            The method registered has one parameter : status        
            """
            self_id = create_callbackdata_id(self)
            # Define the python function
            def DoneCallback_py(taskHandle, status, self_id):
                getattr(get_callbackdata_from_id(self_id),name)(status)
                return 0
            # Transform the python function to a CFunction
            self.DoneCallback_C = DAQmxDoneEventCallbackPtr(DoneCallback_py)
            # Register the function
            self.RegisterDoneEvent(options,self.DoneCallback_C,self_id)

        def AutoRegisterSignalEvent(self, signalID, options, name='SignalCallback'):
            """Register the method named name as the callback function for RegisterSignalEvent
            
            
            With this method you can register a method of the class Task as a callback function. 
            The parameters signalID, options are the same 
            as the DAQmxRegisterSignalEvent parameters  
            
            No parameter are passed to the method      
            """
            self_id = create_callbackdata_id(self)
            # Define the python function
            def SignalCallback_py(taskHandle, signalID, self_id):
                self = get_callbackdata_from_id(self_id)
                getattr(self,name)()
                return 0
            # Transform the python function to a CFunction
            self.SignalCallback_C = DAQmxSignalEventCallbackPtr(SignalCallback_py)
            # Register the function
            self.RegisterSignalEvent(signalID, options, self.SignalCallback_C, self_id)

else:

    class CallbackParent(object):
        def __getattr__(self, name):
            if name in ['AutoRegisterEveryNSamplesEvent', 'AutoRegisterDoneEvent', 'AutoRegisterSignalEvent']:
                raise NotImplementedError('Callback methods are not available')
            return super(CallbackParent, self).__getattr__(name)


class Task(CallbackParent):
    def __init__(self, name=""):
        self.taskHandle = TaskHandle(0)
        DAQmxCreateTask(name,byref(self.taskHandle))

    def __enter__(self):
        return self

    def __exit__(self, *exc_info):
        self.ClearTask()

    def __del__(self):
        """ Clear automatically the task to be able to reallocate resources """ 
        # Clear the task before deleting the object
        # If the task as already been manually cleared, nothing is done
        # This prevent to clear a task that has a Handle attributed to a new task
        # See this example
        # a = Task(), ..., a.ClearTask(), b = Task(), del(a)
        # b has the same taskHandle as a, and deleting a will clear the task of b   
        try: 
            self.ClearTask()
        except Exception:
            pass
    def ClearTask(self):
        if self.taskHandle:
            try:
                DAQmxClearTask(self.taskHandle)
            finally:
                self.taskHandle.value = 0
    def __repr__(self):
        if self.taskHandle:
            return "Task number %d"%self.taskHandle.value
        else:
            return "Invalid or cleared Task"
    # Dynamically creates the method from the task_function_list
    for function_name in task_function_list:
        name = function_name[5:] # remove the DAQmx in front of the name
        func = getattr(DAQmxFunctions, function_name)
        arg_names = function_dict[function_name]['arg_name']
        doc = 'T.%s(%s) -> error.' %(name, ', '.join(arg_names[1:]))
        cmd = """def {0}(self, {1}):
        "{3}"
        return {2}(self.taskHandle, {1})"""
        exec(cmd.format(name, ', '.join(arg_names[1:]), function_name, doc))    
    del function_name, name, func, arg_names, doc


del task_function_list



from PyDAQmx import *
from PyDAQmx.DAQmxCallBack import *
from numpy import zeros

"""This example is a PyDAQmx version of the ContAcq_IntClk.c example
It illustrates the use of callback functions

This example demonstrates how to acquire a continuous amount of 
data using the DAQ device's internal clock. It incrementally stores the data 
in a Python list. 

This example is also an example of the object oriented uses of PyDAQmx
"""

class CallbackTask(Task):
    def __init__(self):
        Task.__init__(self)
        self.data = zeros(1000)
        self.a = []
        self.CreateAIVoltageChan("Dev1/ai0","",DAQmx_Val_RSE,-10.0,10.0,DAQmx_Val_Volts,None)
        self.CfgSampClkTiming("",10000.0,DAQmx_Val_Rising,DAQmx_Val_ContSamps,1000)
        self.AutoRegisterEveryNSamplesEvent(DAQmx_Val_Acquired_Into_Buffer,1000,0)
        self.AutoRegisterDoneEvent(0)
    def EveryNCallback(self):
        read = int32()
        self.ReadAnalogF64(1000,10.0,DAQmx_Val_GroupByScanNumber,self.data,1000,byref(read),None)
        self.a.extend(self.data.tolist())
        print(self.data[0])
        return 0 # The function should return an integer
    def DoneCallback(self, status):
        print("Status",status.value)
        return 0 # The function should return an integer


task=CallbackTask()
task.StartTask()

input('Acquiring samples continuously. Press Enter to interrupt\n')

task.StopTask()
task.ClearTask()



import weakref
from . import DAQmxConfig

""" This package import helper functions to pass data to call back functions

See examples
"""

#if DAQmxConfig.NIDAQmxBase:
#    raise NotImplementedError, 'Call back function are not available with the NIDAQmxBase driver'

# NIDAQmx allow to pass a pointer to a data structure in callback function
# If this function is implemented in Python, we would like this data to be a Python object.
# This object will not (and will not) be used in C but we want to pass a reference to it.
# A workaround is to use the weakref module
# 
# Define a weakref dictionnary to be able to remember objects and retrieve them by ID
_id2obj_dict = weakref.WeakValueDictionary()

def create_callbackdata_id(obj):
    """ 
    Uses the weakref module to create and store a reference to obj
    
    output value: reference to the object
    
    It is not possible to directly uses python object through a Callback function because
    with ctypes there is no pointer to python object.
    This function store in a dictionary a reference to an object
    This object can be retrieved using the get_callbackdata_from_id function
    
    For python object that cannot be weakreferenced, one can creat a dummy class to wrap
    the python object : 
        def MyList(list)
            pass
            
        data = MyList()
        id = create_callbackdata_id(data)
        
    """
    oid = id(obj)
    _id2obj_dict[oid] = obj
    return oid

def get_callbackdata_from_id(oid):
    """Retrieve an object stored using create_callbackdata_id    
    """
    return _id2obj_dict[oid]



import re
import sys
from ctypes import *
import warnings

from . import DAQmxConfig
from .DAQmxTypes import *
from .DAQmxConstants import constant_list
from . import DAQmxConstants

class DAQException(Exception):
    """Exception raised from the NIDAQ.

    Attributes:
        error -- Error number from NI
        message -- explanation of the error
    """
    code = None
    def __init__(self, message, fname):
        self.message = message
        self.fname = fname
    @property
    def error(self):
        """ Returns the error code"""
        # for compatibility with older version
        return self.code
    def __str__(self):
        return self.message + '\n in function '+self.fname


class DAQError(DAQException):
    pass

error_by_number = {}

class DAQWarning(DAQException,Warning):
    pass

warning_by_number = {}

error_list = ["DAQError", "DAQWarning", "DAQException"]

for name in constant_list:
    if name.startswith('DAQmxError'):
        errname = name[10:]
        code = getattr(DAQmxConstants, name)
        error_by_number[code] = globals()[errname + 'Error'] = type(errname + 'Error', (DAQError,), dict(code=code))
        error_list.append(errname + 'Error')
    elif name.startswith('DAQmxWarning'):
        errname = name[12:]
        code = getattr(DAQmxConstants, name)
        warning_by_number[code] = globals()[errname + 'Warning'] = type(errname + 'Warning', (DAQWarning,), dict(code=code))
        error_list.append(errname + 'Warning')

def catch_error_default(f):
    def mafunction(*arg):
        error = f(*arg)
        if error<0:
            errBuff = create_string_buffer(2048)
            DAQmxGetExtendedErrorInfo(errBuff,2048)
            exception_class = error_by_number.get(error, DAQError)
            raise exception_class(errBuff.value.decode("utf-8"), f.__name__)
        elif error>0:
            errBuff = create_string_buffer(2048)
            DAQmxGetErrorString(error, errBuff, 2048);
            exception_class = warning_by_number.get(error, DAQWarning)
            warnings.warn(exception_class(errBuff.value.decode("utf-8"), f.__name__))
        return error
    return mafunction

#def catch_error_DAQmxGetSys(f):
#    """ Catch error if the first argument is not None"""
#    default_f = catch_error_default(f)
#    def mafunction(*args):
#        if args[0] is not None:
#            return default_f(*args)
#        return f(*args)
#    return mafunction

def catch_error_buffer(f, data_pos):
    """ Catch error only if the data arg is not None"""
    default_f = catch_error_default(f)
    def mafunction(*args):
        if args[data_pos] is not None:
            return default_f(*args)
        return f(*args)
    return mafunction

def catch_error(f, name, arg_list, arg_name):
#    if name.startswith("DAQmxGetSys"):
#        return catch_error_DAQmxGetSys(f)
    if 'data' in arg_name and 'bufferSize' in arg_name:
        return catch_error_buffer(f, arg_name.index('data'))
    return catch_error_default(f)


def _add_keywords(arg_name):
    """ This function is used to create a decorator that add arg_name keywords to a function"""
    s = """def add_keywords_decorator(f):
    def function({0}):return f({0})
    return function"""
    local_dct = {}
    exec(s.format(', '.join(arg_name)), local_dct)
    return local_dct['add_keywords_decorator']


#if lib_name is not None:
#    if sys.platform.startswith('win'):        
#        DAQlib = windll.LoadLibrary(lib_name)
#        DAQlib_variadic = cdll.LoadLibrary(lib_name)
#    elif sys.platform.startswith('linux'):
#        DAQlib = cdll.LoadLibrary(lib_name)
#        DAQlib_variadic = DAQlib
#else: # NIDAQmx is not installed on the machine. Use a dummy library
#    class _nothing():
#        def __getattr__(self, name):
#            return lambda *args:0
#    DAQlib = _nothing()
#    DAQlib_variadic = DAQlib
DAQlib, DAQlib_variadic = DAQmxConfig.get_lib()


######################################
# Array
######################################
#Depending whether numpy is install or not, 
#the function array_type is defined to return a numpy array or
#a ctype POINTER
try:
    import numpy
except ImportError:
    def array_type(string):
        return eval('POINTER('+string+')')
else:
    # Type conversion for numpy
    def numpy_conversion(string):
        """ Convert a type given by a string to a numpy type

        """
        #This function uses the fact that the name are the same name, 
        #except that numpy uses lower case
        return eval('numpy.'+string.lower())
    def array_type(string):
        """ Returns the array type required by ctypes when numpy is used """
        return numpy.ctypeslib.ndpointer(dtype = numpy_conversion(string),
                                         flags=('C_CONTIGUOUS','WRITEABLE'))

################################
#Read the .h file and convert the function for python
################################
include_file = open(DAQmxConfig.dot_h_file,'r') #Open NIDAQmx.h file

################################
# Regular expression to parse the NIDAQmx.h file
# Almost all the function define in NIDAQmx.h file are imported
################################

# Each regular expression is associated with a ctypes type and a number giving the
# group in which the name of the variable is defined


function_parser = re.compile(r'__CFUNC.* (DAQ\S+)\s*\((.*)\);')
const_char = re.compile(r'(const char)\s*([^\s]*)\[\]')
string_type = '|'.join(['int8','uInt8','int16','uInt16','int32','uInt32','float32','float64','int64','uInt64','bool32','TaskHandle'])


type_list = ['int8','uInt8','int16','uInt16','int32','uInt32','float32','float64',
    'int64','uInt64','bool32','TaskHandle']
type_list_array = ['int8','uInt8','int16','uInt16','int32','uInt32','float32','float64',
        'int64','uInt64']


# Each regular expression is associated with a ctypes type and a number giving the
# group in which the name of the variable is defined
const_char = [(re.compile(r'(const char)\s*([^\s]*)\[\]'), CtypesString() ,2)]
simple_type = [(re.compile('('+_type+r')\s*([^\*\[]*)\Z'),eval(_type),2)
     for _type in type_list]
pointer_type = [(re.compile('('+_type+r')\s*\*([^\*]*)\Z'),
        eval('POINTER('+_type+')'),2) for _type in type_list]
pointer_type_array = [(re.compile('('+_type+r')\s*((?:readArray|writeArray).*)\[\]\Z'),
    array_type(_type),2) for _type in type_list_array]
pointer_type_2 = [(re.compile('('+_type+r')\s*([^\s]*)\[\]\Z'),
        eval('POINTER('+_type+')'),2) for _type in type_list]

const_char_etoile = [(re.compile(r'(const char)\s*\*([^\*]*)\Z'), CtypesString(), 2)] # match "const char * name"
char_etoile = [(re.compile(r'(char)\s*\*([^\*]*)\Z'), c_char_p, 2)] # match "char * name"
void_etoile = [(re.compile(r'(void)\s*\*([^\*]*)\Z'), c_void_p, 2)] # match "void * name"
char_array = [(re.compile(r'(char)\s*([^\s]*)\[\]'), c_char_p,2)] # match "char name[]"
call_back_A = [(re.compile(r'(DAQmxEveryNSamplesEventCallbackPtr)\s*([^\s]*)'),DAQmxEveryNSamplesEventCallbackPtr ,2)]
call_back_B = [(re.compile(r'(DAQmxDoneEventCallbackPtr)\s*([^\s]*)'),DAQmxDoneEventCallbackPtr,2)]
call_back_C = [(re.compile(r'(DAQmxSignalEventCallbackPtr)\s*([^\s]*)'),DAQmxSignalEventCallbackPtr,2)]

variadic = [(re.compile(r'\.\.\.'), "variadic", None)]

# Create a list with all regular expressions
c_to_ctype_map = []
for l in [const_char, simple_type, pointer_type, pointer_type_array, pointer_type_array,
        pointer_type_2, const_char_etoile, char_etoile, void_etoile,char_array,
          call_back_A, call_back_B, call_back_C, variadic]:
    c_to_ctype_map.extend(l)



# The list of all function 
# function_dict: the keys are function name, the value is a dictionnary 
# with 'arg_type' and 'arg_name', the type and name of each argument 
function_list = [] 
function_dict = {} 

def _define_function(name, arg_list, arg_name):
    if "variadic" in arg_list:
        _define_variadic_function(name, arg_list, arg_name)
        return
    # Fetch C function and apply argument checks
    cfunc = getattr(DAQlib, name, None)
    if cfunc is None:
        warnings.warn('Unable to load {0}'.format(name))
        return
    if DAQmxConfig.NIDAQmxBase and 'Base' in name :
        name = name[:5]+name[9:]  
    setattr(cfunc, 'argtypes', arg_list)
    # Create error-raising wrapper for C function and add to module's dict
    func = _add_keywords(arg_name)(catch_error(cfunc, name, arg_list, arg_name))
    func.__name__ = name
    func.__doc__ = '%s(%s) -> error.' % (name, ', '.join(arg_name))
    globals()[name] = func
    # Record details of function
    function_dict[name] = {'arg_type':arg_list, 'arg_name':arg_name}

def _define_variadic_function(name, arg_list, arg_name):
    cfunc = getattr(DAQlib_variadic, name)
    if DAQmxConfig.NIDAQmxBase and 'Base' in name :
        name = name[:5]+name[9:]    
    func = _add_keywords(arg_name)(catch_error(cfunc, name, arg_list, arg_name))
    func.__name__ = name
    func.__doc__ = '%s(%s) -> error.' % (name, ', '.join(arg_name))
    globals()[name] = func
    function_dict[name] = {'arg_type':arg_list, 'arg_name':arg_name}


argsplit = re.compile(', |,')
for line in include_file:
    fn_match = function_parser.search(line[:-1])
    if fn_match and not line.startswith('//'):
        name = fn_match.group(1)
        function_list.append(name)
        arg_string = fn_match.group(2)
        arg_list=[]
        arg_name = []
        for arg in argsplit.split(arg_string): # Almost everywhere there is a space after the comma
            for (reg_expr, new_type, group_nb) in c_to_ctype_map:
                reg_expr_result = reg_expr.search(arg)
                if reg_expr_result is not None:
                    if new_type=="variadic":
                        arg_list.append(new_type)
                        arg_name.append("*args")
                        break
                    arg_list.append(new_type)
                    arg_name.append(reg_expr_result.group(group_nb))
                    break # break the for loop    
        _define_function(name, arg_list, arg_name)

include_file.close()




# Clean private functions from namespace
del _define_function



import sys
import platform
import os
import ctypes
from ctypes.util import find_library
import platform


dot_h_file = None

NIDAQmxBase = False

if sys.platform.startswith('win') or sys.platform.startswith('cli'):
    # Full path of the NIDAQmx.h file
    # Default location on Windows XP and Windows 7
    dot_h_dir_x86, dot_h_dir_x64, dot_h_dir_from_reg = [], [], []
    if 'PROGRAMFILES(X86)' in os.environ:
        dot_h_dir_x86 = [os.path.join(os.environ['PROGRAMFILES(X86)'],
                                      r'National Instruments\NI-DAQ\DAQmx ANSI C Dev\include\NIDAQmx.h'),
                         os.path.join(os.environ['PROGRAMFILES(X86)'],
                                      r'National Instruments\Shared\ExternalCompilerSupport\C\include\NIDAQmx.h')]
    if 'PROGRAMFILES' in os.environ:
        dot_h_dir_x64 = [os.path.join(os.environ['PROGRAMFILES'],
                                      r'National Instruments\NI-DAQ\DAQmx ANSI C Dev\include\NIDAQmx.h'),
                         os.path.join(os.environ['PROGRAMFILES'],
                                      r'National Instruments\Shared\ExternalCompilerSupport\C\include\NIDAQmx.h')]
    try:
        import winreg
    except ImportError:
        winreg = None
    if winreg:
        dirs_from_reg = []
        keys = ((r"SOFTWARE\WOW6432Node\National Instruments\Common\Installer", r"NIDIR"),
                    (r"SOFTWARE\National Instruments\Common\Installer", r"NIDIR64"))
        reg = winreg.ConnectRegistry(None,winreg.HKEY_LOCAL_MACHINE)
        for key in keys:
            try:
                key_open = winreg.OpenKey(reg, key[0])
                var = winreg.QueryValueEx(key_open, key[1])
                dirs_from_reg.append(var[0])
            except FileNotFoundError:
                pass
        for dir in dirs_from_reg:
            dot_h_dir_from_reg.append(os.path.join(dir, r"NI-DAQ\DAQmx ANSI C Dev\include\NIDAQmx.h"))
            dot_h_dir_from_reg.append(os.path.join(dir, r"Shared\ExternalCompilerSupport\C\include\NIDAQmx.h"))

    dot_h_file = None
    for file in dot_h_dir_x64 + dot_h_dir_x86 + dot_h_dir_from_reg:
        if os.path.exists(file): dot_h_file = file

    # Name (and eventually path) of the library
    # Default on Windows is nicaiu
    lib_name = "nicaiu"
    if 'iron' in platform.python_implementation().lower():
        def get_lib():
            lib_name = "nicaiu"
            DAQlib = ctypes.windll.nicaiu
            DAQlib_variadic = ctypes.cdll.nicaiu
            return DAQlib, DAQlib_variadic
    else:
        def get_lib():
            lib_name = "nicaiu"
            DAQlib = ctypes.windll.LoadLibrary(lib_name)
            DAQlib_variadic = ctypes.cdll.LoadLibrary(lib_name)        
            return DAQlib, DAQlib_variadic


elif sys.platform.startswith('linux'):
    # On linux you can use the command find_library('nidaqmx')

    lib_name = find_library('nidaqmx')
    if lib_name is not None:
        for dot_h_file in ['/usr/include/NIDAQmx.h', 
                           '/usr/local/natinst/nidaqmx/include/NIDAQmx.h']:
            if os.path.exists(dot_h_file):
                break
        else:
            dot_h_file = None
        def get_lib():
            lib_name = find_library('nidaqmx')
            DAQlib = ctypes.cdll.LoadLibrary(lib_name)
            DAQlib_variadic = DAQlib
            return DAQlib, DAQlib_variadic



    lib_name = find_library('nidaqmxbase')
    if lib_name is not None:

        dot_h_file = '/usr/local/natinst/nidaqmxbase/include/NIDAQmxBase.h'
        NIDAQmxBase = True
        def get_lib():
            lib_name = find_library('nidaqmxbase')
            ctypes.CDLL(find_library('lvrtdark'), mode=ctypes.RTLD_GLOBAL)
            DAQlib = ctypes.cdll.LoadLibrary(lib_name)
            DAQlib_variadic = DAQlib
            return DAQlib, DAQlib_variadic

elif sys.platform.startswith('darwin'):
    lib_name = find_library('nidaqmxbase')
    if lib_name is not None:
        dot_h_file = '/Applications/National Instruments/NI-DAQmx Base/includes/NIDAQmxBase.h'
        NIDAQmxBase = True
        def get_lib():
            lib_name = find_library('nidaqmxbase')
            DAQlib = ctypes.cdll.LoadLibrary(lib_name)
            DAQlib_variadic = DAQlib
            return DAQlib, DAQlib_variadic


# If the DAQmxConfigTest has been imported, then uses the value from this file
# This can be used to try different version or compile the module on a plateform where 
# DAQmx is not installed
if "DAQmxConfigTest" in list(sys.modules.keys()):
    from DAQmxConfigTest import *
    if lib_name is None:
        def get_lib():
            class _nothing():
                def __getattr__(self, name):
                    return lambda *args:0
            DAQlib = _nothing()
            DAQlib_variadic = DAQlib
            return DAQlib, DAQlib_variadic

if dot_h_file is None or not os.path.exists(dot_h_file):
    raise NotImplementedError("Location of niDAQmx library and include file unknown on %s - if you find out, please let the PyDAQmx project know" % (sys.platform))




from .ContinuousPulseTrainGeneration import ContinuousPulseTrainGeneration
from .MultiChannelAnalogInput import MultiChannelAnalogInput
from .callback_Task_synchronous import CallbackTaskSynchronous
from .callback_test_unregister import CallbackWithUnregister



from PyDAQmx import *
from PyDAQmx.DAQmxCallBack import *
from numpy import zeros

"""This example is a PyDAQmx version of the ContAcq_IntClk.c example
It illustrates the use of callback functions

This example demonstrates how to acquire a continuous amount of 
data using the DAQ device's internal clock. It incrementally store the data 
in a Python list. 

This example is also an example of the object oriented uses of PyDAQmx
"""



# one cannot create a weakref to a list directly
# but the following works well
class MyList(list):
    pass

# list where the data are stored
data = MyList()
id_a = create_callbackdata_id(data)



# Define two Call back functions
def EveryNCallback_py(taskHandle, everyNsamplesEventType, nSamples, callbackData_ptr):
    callbackdata = get_callbackdata_from_id(callbackData_ptr)
    read = int32()
    data = zeros(1000)
    DAQmxReadAnalogF64(taskHandle,1000,10.0,DAQmx_Val_GroupByScanNumber,data,1000,byref(read),None)
    callbackdata.extend(data.tolist())
    print("Acquired total %d samples"%len(callbackdata))
    return 0 # The function should return an integer

# Convert the python function to a CFunction      
EveryNCallback = DAQmxEveryNSamplesEventCallbackPtr(EveryNCallback_py)

def DoneCallback_py(taskHandle, status, callbackData):
    print("Status",status.value)
    return 0 # The function should return an integer

# Convert the python function to a CFunction      
DoneCallback = DAQmxDoneEventCallbackPtr(DoneCallback_py)


# Beginning of the script

#DAQmxResetDevice('dev1')

task=Task()
task.CreateAIVoltageChan("Dev1/ai0","",DAQmx_Val_RSE,-10.0,10.0,DAQmx_Val_Volts,None)
task.CfgSampClkTiming("",10000.0,DAQmx_Val_Rising,DAQmx_Val_ContSamps,1000)

task.RegisterEveryNSamplesEvent(DAQmx_Val_Acquired_Into_Buffer,1000,0,EveryNCallback,id_a)
task.RegisterDoneEvent(0,DoneCallback,None)

task.StartTask()

input('Acquiring samples continuously. Press Enter to interrupt\n')

task.StopTask()
task.ClearTask()



import PyDAQmx
from PyDAQmx import Task, DAQmxResetDevice, int32
from ctypes import byref
import numpy


class AIParameters(object):
    limits = (-10, 10)
    physicalChannel = ["/Dev1/ai0"]

    def __init__(self, sample_rate, sample_number, channels=None, limits=None):
        self.sampleRate = sample_rate
        self.sampleNumber = sample_number
        if limits is None:
            limits = self.limits
        self.limit_inf= limits[0]
        self.limit_sup= limits[1]
        if channels is not None:
            if type(channels) is str:
                physicalChannel = [channels]
            self.physicalChannel = channels

    @property
    def device_name(self):
        device_name = self.physicalChannel[0].split('/')[0]
        if device_name == '' :
            device_name = self.physicalChannel[0].split('/')[1]
        return device_name

class Trigger(object):
    def __init__(self, terminal):
        self.terminal = terminal

class RisingTrigger(Trigger):
    direction = PyDAQmx.DAQmx_Val_Rising
         
class FallingTrigger(Trigger):
    direction = PyDAQmx.DAQmx_Val_Falling


class AIVoltageChan(Task):
    def __init__(self, ai_param, reset=True, terminalConfig=PyDAQmx.DAQmx_Val_RSE, trigger=None):
        if reset:
            DAQmxResetDevice(ai_param.device_name)
        super(AIVoltageChan, self).__init__()
        self.sampleNumber = ai_param.sampleNumber
        self.sampleRate = ai_param.sampleRate
        self.limit_inf = ai_param.limit_inf
        self.limit_sup = ai_param.limit_sup
        self.physicalChannel = ai_param.physicalChannel
        self.numberOfChannel = len(ai_param.physicalChannel)
        if isinstance(terminalConfig, str):
            terminalConfig = getattr(PyDAQmx, terminalConfig)
        self.terminalConfig = terminalConfig
        self.trigger = trigger
        self.configure()
    def configure(self):
        channel_string = ','.join(self.physicalChannel)
        self.CreateAIVoltageChan(channel_string,"",self.terminalConfig,
                                 self.limit_inf,self.limit_sup,
                                 PyDAQmx.DAQmx_Val_Volts,None)
    def start(self):
        n = self.sampleNumber
        sampleRate = self.sampleRate
        self.CfgSampClkTiming("",sampleRate,PyDAQmx.DAQmx_Val_Rising,PyDAQmx.DAQmx_Val_FiniteSamps,n)
        if self.trigger is not None:
            self.CfgDigEdgeRefTrig(self.trigger.terminal,self.trigger.direction,10)
        self.StartTask()
    def read(self):
        n = self.sampleNumber
        data = numpy.zeros((n,self.numberOfChannel), dtype=numpy.float64)
        read = int32()
        self.ReadAnalogF64(n,10.0,PyDAQmx.DAQmx_Val_GroupByScanNumber,data,n*self.numberOfChannel,byref(read),None)
        return data
    def stop(self):
        self.StopTask()
    def wait(self, timeout=10):
        self.WaitUntilTaskDone(timeout)


if __name__=="__main__":
    ai = AIVoltageChan(ai_param=AIParameters(100000, 10000, ['/dev1/ai0', '/dev1/ai1']), 
                    terminalConfig="DAQmx_Val_PseudoDiff", 
                    trigger=RisingTrigger('/dev1/PFI0'))
    ai.start()
    ai.wait()
    ai.read()
    ai.stop()



# coding= latin-1

from PyDAQmx.DAQmxFunctions import *
from PyDAQmx.DAQmxConstants import *

class ContinuousPulseTrainGeneration():
    """ Class to create a continuous pulse train on a counter
    
    Usage:  pulse = ContinuousTrainGeneration(period [s],
                duty_cycle (default = 0.5), counter (default = "dev1/ctr0"),
                reset = True/False)
            pulse.start()
            pulse.stop()
            pulse.clear()
    """
    def __init__(self, period=1., duty_cycle=0.5, counter="Dev1/ctr0", reset=False):
        if reset:
            DAQmxResetDevice(counter.split('/')[0])
        taskHandle = TaskHandle(0)
        DAQmxCreateTask("",byref(taskHandle))
        DAQmxCreateCOPulseChanFreq(taskHandle,counter,"",DAQmx_Val_Hz,DAQmx_Val_Low,
                                                                   0.0,1/float(period),duty_cycle)
        DAQmxCfgImplicitTiming(taskHandle,DAQmx_Val_ContSamps,1000)
        self.taskHandle = taskHandle
    def start(self):
        DAQmxStartTask(self.taskHandle)
    def stop(self):
        DAQmxStopTask(self.taskHandle)
    def clear(self):
        DAQmxClearTask(self.taskHandle)


if __name__=="__main__":
    pulse_gene1 = ContinuousPulseTrainGeneration(1.,0.5, "dev1/ctr0", reset=True)
    pulse_gene1.start()
    a = input("Generating pulse train. Press Enter to interrupt\n")
    pulse_gene1.stop()
    pulse_gene1.clear()
   





import os
from types import ModuleType
import unittest


from . import test_MultiChannelAnalogInput

#suite = unittest.TestSuite([test_MultiChannelAnalogInput.suite])


# This function is called by unittest.main
# Do no delete !!!!
def load_tests(loader, standard_tests, pattern):
    for name, elm in list(globals().items()):
        if name.startswith('test') and isinstance(elm, ModuleType):        
            suite = loader.loadTestsFromModule(elm)
            standard_tests.addTests(suite)
    return standard_tests




import sys

from ctypes import *
import ctypes
from .DAQmxConstants import DAQmx_copyright_year

# New types definitions
# Correspondance between the name used in the NiDAQmx.h file and ctypes
int8 = c_byte
uInt8 = c_ubyte
int16 = c_short
uInt16 = c_ushort 
int32 = c_int 
uInt32 = c_uint 
float32 = c_float 
float64 = c_double 
int64 =c_longlong 
uInt64 = c_ulonglong 
bool32 = uInt32 
if DAQmx_copyright_year<2010:
    TaskHandle = uInt32
else:
    TaskHandle = c_void_p
CalHandle = uInt32

# CFUNCTYPE defined in NIDAQmx.h
DAQmxEveryNSamplesEventCallbackPtr = CFUNCTYPE(int32, TaskHandle, int32, uInt32, c_void_p)
DAQmxDoneEventCallbackPtr = CFUNCTYPE(int32, TaskHandle, int32, c_void_p)
DAQmxSignalEventCallbackPtr = CFUNCTYPE(int32, TaskHandle, int32, c_void_p)

class CtypesString(object):
    """ Argtypes to automatically convert str to bytes in Python3 """
    def from_param(self, param):
        if sys.version_info >= (3,):
            if isinstance(param, str):
                param = param.encode('ascii')
        return c_char_p(param)







# -*- coding: utf-8 -*-

from . import DAQmxConfig

from .DAQmxTypes import *
from .DAQmxConstants import *
from .DAQmxFunctions import *
from .Task import Task

from . import DAQmxConstants
from . import DAQmxFunctions

all_types = ['int8', 'uInt8', 'int16', 'uInt16', 'int32', 'uInt32', 'float32', 'float64', 'int64', 'uInt64', 'bool32',
        'TaskHandle', 'CalHandle', 'DAQmxEveryNSamplesEventCallbackPtr', 'DAQmxDoneEventCallbackPtr', 'DAQmxSignalEventCallbackPtr', 'CtypesString']


__all__ = DAQmxConstants.constant_list + list(DAQmxFunctions.function_dict.keys()) + ['Task'] + all_types + DAQmxFunctions.error_list

for name in DAQmxConstants.constant_list + list(DAQmxFunctions.function_dict.keys()):
    if name.startswith('DAQmx_'):
        new_name = name[6:]
    elif name.startswith('DAQmx'):
        new_name = name[5:]
    globals()[new_name] = globals()[name]

##### WARNING, version should also be modified in the setup.py
__version_info__ = (1, 4, 7)
__version__ = '.'.join(str(num) for num in __version_info__)

__author__ =u'Pierre Cladé'



# -*- coding: utf-8 -*-
import PyDAQmx as daqmx
from PyDAQmx.DAQmxCallBack import *
import numpy as np
from ctypes import cast

# Sample code to test changing the callback function for a daqmx acquisition
# task. 
#
# Author: Erik Matlin, 9/9/2014
# Modified by Pierre Cladé 23/9/2014

class CallbackWithUnregister(object):
    def __init__(self, device, nSamples=1000):
        self.task = daqmx.Task()
        self.task.EveryNCallback1 = self.EveryNCallback1
        self.task.EveryNCallback2 = self.EveryNCallback2

        self.task.CreateAIVoltageChan("/".join((device,'ai0')), "a", 
            daqmx.DAQmx_Val_Cfg_Default, -10, 10, daqmx.DAQmx_Val_Volts, None)
            

    def start(self, nSamples, callbackname):
        # configure timing
        self.task.CfgSampClkTiming("", 10000, daqmx.DAQmx_Val_Rising, 
            daqmx.DAQmx_Val_ContSamps, nSamples)
        self.nSamples = nSamples
        
        self.task.AutoRegisterEveryNSamplesEvent(daqmx.DAQmx_Val_Acquired_Into_Buffer, nSamples, 0, name=callbackname)                
        self.task.StartTask()
    
    def stop(self):
        self.task.StopTask()
        
    def EveryNCallback1(self):
        read = daqmx.int32(0)
        data = np.zeros(self.nSamples)
        self.task.ReadAnalogF64(self.nSamples, 1.0, 
            daqmx.DAQmx_Val_GroupByScanNumber, data, data.size, 
            daqmx.byref(read), None)
        
    def EveryNCallback2(self):
        read = daqmx.int32(0)
        data = np.zeros(self.nSamples)
        self.task.ReadAnalogF64(self.nSamples, 1.0, 
            daqmx.DAQmx_Val_GroupByScanNumber, data, data.size, 
            daqmx.byref(read), None)


if __name__ == '__main__':
    from time import sleep
    b = CallbackWithUnregister("TestDevice")
    for i,nSamples in enumerate([1000, 2000, 5000]):
        func_name = 'EveryNCallback{0}'.format([1,2][i%2])
        b.start(nSamples, func_name)
        print("started!")
        sleep(.5)
        b.stop()
        print("stop")



# -*- coding: utf-8 -*-
from PyDAQmx import *
from PyDAQmx.DAQmxCallBack import *
from numpy import zeros
from ctypes import byref
import threading

"""This example is a PyDAQmx version of the ContAcq_IntClk.c example,
illustrating the use of callback functions and the object-oriented Task
interface. It also demonstrates a method of synchronising a main thread with
the asynchronous EveryNCallback.

This example demonstrates how to acquire a continuous amount of data using the
DAQ device's internal clock.
"""

class CallbackTaskSynchronous(Task):
    def __init__(self, dev_name=None, data_len=1000):
        Task.__init__(self)
        if dev_name is None:
            dev_name = "Dev1"
        self._data = zeros(1000)
        self.read = int32()
        self.CreateAIVoltageChan(dev_name+"/ai0","",DAQmx_Val_RSE,-10.0,10.0,DAQmx_Val_Volts,None)
        self.CfgSampClkTiming("",100000.0,DAQmx_Val_Rising,DAQmx_Val_ContSamps,1000)
        self.AutoRegisterEveryNSamplesEvent(DAQmx_Val_Acquired_Into_Buffer,1000,0)
        self.AutoRegisterDoneEvent(0)
        self._data_lock = threading.Lock()
        self._newdata_event = threading.Event()
    def EveryNCallback(self):
        with self._data_lock:
            self.ReadAnalogF64(1000,10.0,DAQmx_Val_GroupByScanNumber,self._data,1000,byref(self.read),None)
            self._newdata_event.set()
        return 0 # The function should return an integer
    def DoneCallback(self, status):
        print("Status",status.value)
        return 0 # The function should return an integer
    def get_data(self, blocking=True, timeout=None):
        if blocking:
            if not self._newdata_event.wait(timeout):
                raise ValueError("timeout waiting for data from device")
        with self._data_lock:
            self._newdata_event.clear()
            return self._data.copy()


if __name__=="__main__":
    task=CallbackTaskSynchronous()
    task.StartTask()

    print("Acquiring 10 * 1000 samples in continuous mode.")
    for _ in range(10):
        task.get_data(timeout=10.0)
        print("Acquired %d points" % task.read.value)

    task.StopTask()
    task.ClearTask()



# coding= latin-1
""" Acquisition of multiple channel

This is a example of using classes. This example is obsolete as it does not use the Task object

"""

import numpy

from PyDAQmx.DAQmxFunctions import *
from PyDAQmx.DAQmxConstants import *
from PyDAQmx import TaskHandle, int32
from ctypes import byref

class MultiChannelAnalogInput():
    """Class to create a multi-channel analog input
    
    Usage: AI = MultiChannelInput(physicalChannel)
        physicalChannel: a string or a list of strings
    optional parameter: limit: tuple or list of tuples, the AI limit values
                        reset: Boolean
    Methods:
        read(name), return the value of the input name
        readAll(), return a dictionary name:value
    """
    def __init__(self,physicalChannel, limit = None, reset = False):
        if type(physicalChannel) == type(""):
            self.physicalChannel = [physicalChannel]
        else:
            self.physicalChannel  =physicalChannel
        self.numberOfChannel = physicalChannel.__len__()
        if limit is None:
            self.limit = dict([(name, (-10.0,10.0)) for name in self.physicalChannel])
        elif type(limit) == tuple:
            self.limit = dict([(name, limit) for name in self.physicalChannel])
        else:
            self.limit = dict([(name, limit[i]) for  i,name in enumerate(self.physicalChannel)])           
        if reset:
            DAQmxResetDevice(physicalChannel[0].split('/')[0] )
    def configure(self):
        # Create one task handle per Channel
        taskHandles = dict([(name,TaskHandle(0)) for name in self.physicalChannel])
        for name in self.physicalChannel:
            DAQmxCreateTask("",byref(taskHandles[name]))
            DAQmxCreateAIVoltageChan(taskHandles[name],name,"",DAQmx_Val_RSE,
                                     self.limit[name][0],self.limit[name][1],
                                     units=DAQmx_Val_Volts,customScaleName=None)
        self.taskHandles = taskHandles
    def readAll(self):
        return dict([(name,self.read(name)) for name in self.physicalChannel])
    def read(self,name = None):
        if name is None:
            name = self.physicalChannel[0]
        taskHandle = self.taskHandles[name]                    
        DAQmxStartTask(taskHandle)
        data = numpy.zeros((1,), dtype=numpy.float64)
#        data = AI_data_type()
        read = int32()
        DAQmxReadAnalogF64(taskHandle,1,10.0,DAQmx_Val_GroupByChannel,data,1,byref(read),None)
        DAQmxStopTask(taskHandle)
        return data[0]


if __name__ == '__main__':
    multipleAI = MultiChannelAnalogInput(["Dev1/ai2","Dev1/ai1"])
    multipleAI.configure()



import unittest

from ..MultiChannelAnalogInput import MultiChannelAnalogInput

class Test(unittest.TestCase):
    def test(self):
        multipleAI = MultiChannelAnalogInput(["TestDevice/ai2","TestDevice/ai1"],  reset=True)
        multipleAI.configure()
        multipleAI.read("TestDevice/ai2")


suite = unittest.TestSuite([unittest.TestLoader().loadTestsFromTestCase(Test)])




# -*- coding: utf-8 -*-
# This file should be compatible with both Python 2 and Python 3
from __future__ import print_function, unicode_literals

import sys

from setuptools import setup
from setuptools.command.test import test as TestCommand

##### WARNING version string should be modified also in the __init__.py
version = '1.4.7'

import os
directory = os.path.split(os.path.realpath(__file__))[0]
os.chdir(directory)

class Test(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = ['PyDAQmx']
        self.test_suite = True

    def run_tests(self):
        import unittest

        try:  
            import PyDAQmx
        except NotImplementedError:
            import DAQmxConfigTest

        import PyDAQmx

        print("Functions and constants are imported from : " + PyDAQmx.DAQmxConfig.dot_h_file)

        if PyDAQmx.DAQmxConfig.lib_name is None:
            print('DAQmx is not installed. PyDAQmx is using a dummy library for tests')
            unittest.main('PyDAQmxTest.test_without_daqmx', argv=[unittest.__file__])
        else:
            print("The library is : " + PyDAQmx.DAQmxConfig.lib_name)
            if PyDAQmx.DAQmxConfig.NIDAQmxBase:
                unittest.main('PyDAQmxTest.test_daqmx_base', argv=[unittest.__file__])
            else:
                unittest.main('PyDAQmxTest.test_full', argv=[unittest.__file__])    

class TestExample(TestCommand):
    user_options = [(b'example=', b'm', b"Test example file name")]
    example = 'DEFAULT'
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import unittest

        import PyDAQmx
        import PyDAQmxTest
        execfile(os.path.join(os.path.dirname(PyDAQmxTest.__file__) , self.example))


if sys.version_info >= (3,):
    packages = ["PyDAQmx", 'PyDAQmx.example', 'PyDAQmx.example.test']
else:
    packages = [b"PyDAQmx", b'PyDAQmx.example', b'PyDAQmx.example.test']



long_description = """\
Overview
========

This package allows users to use data acquisition hardware from `National 
Instrument`_ with python. It makes an interface between the NIDAQmx driver 
and python. It currently works only on Windows.

The package is not an open source driver from NI acquisition hardware. You first need to install the driver provided by NI

Compare to similar package, the PyDAQmx module is a full interface to
the NIDAQmx ansi C driver. It imports all the functions from the
driver and also imports all the predefined constants. This provided an
almost one to one match between C and python code. Examples using
callback functions are provided.

A more convenient Object oriented interface is provided, where the mecanism 
of taskHandle in C is replace with a Task object.

**Detailed information** about this package can be found on its `main
website`_.



Installation
============

You need first to install the NI DAQmx driver which is provided with your 
data-acquisition hardware. Please verify that you have install together with 
the driver the C API (which should be the case by default). 

To install PyDAQmx, download the package and run the command:: 

  python setup.py install

You can also directly move the PyDAQmx directory to a location
that Python can import from (directory in which scripts 
using PyDAQmx are run, etc.)
 

Contact
=======

Please send bug reports or feedback to `{auth_name}`_.

Version history
===============
Main changes:

* 1.4.7 Compatible with python 3.13
* 1.4.6 Use windows registry to search for files
* 1.4.5 All files compatible with Python 2 and Python 3 (remove 2to3)
* 1.4.4 New location introduced by DAQmx 19
* 1.4.3 Support for Centos 7
* 1.4.2 Proper version string
* 1.4.1 NIDAQmx Base supported on 64bits linux
* 1.4 Many small improvements
* 1.3.2 bug fix
* 1.3.1 With python 3, strings (unicode) can be use as arguments
* 1.3 PyDAQmx supports both the NIDAQmx and NIDAQmxBase drivers
* 1.2.5.2 Bug in version 1.2.5 corrected (Task were not working)
* 1.2.5.1 Add keywords to all the functions (version 1.2.5 is not working with python 3)
* 1.2.4 NIDAQmx functions of the 2011 et 2012 NIDAQmx are imported properly
* 1.2.3 DAQmxAddNetworkDevice is now working
* 1.2.2 The package is working with python 3 using 2to3
* 1.2.1 Add doc string to the DAQmxFunctions
* 1.2 Support of callback function
* 1.1 Add linux support

Version 1.4
===========

Improvements are the following : 

* Each error has a specificc subclass
* Constants can be loaded without the prefix : PyDAQmx.Val_Cfg_Default
* Throw warnings as warnings
* Improve unittest. 

Version 2
=========

A new version of PyDAQmx will is in the dev2 branch on github. PyDAQmx will follow the pep8 naming convention (and will be called pydaqmx). A legacy mode will still be available. 

.. _National Instrument: http://www.ni.com
.. _{auth_name}: mailto:pierre.clade@spectro.jussieu.fr
.. _main website: http://pythonhosted.org/PyDAQmx/
"""

setup_parameters = dict(version=version,
      name = "PyDAQmx",
      author_email="pierre.clade@spectro.jussieu.fr",
      maintainer_email="pierre.clade@spectro.jussieu.fr",
      url='http://pythonhosted.org/PyDAQmx/',
      license='''\
This software can be used under one of the following two licenses: \
(1) The BSD license. \
(2) Any other license, as long as it is obtained from the original \
author.''',
      description='Interface to the National Instruments PyDAQmx driver',
      keywords=['DAQmx', 'National Instrument', 'Data Acquisition','nidaq','nidaqmx'],
      classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Other Audience',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Physics',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules'], 
     packages=packages, 
#     use_2to3=True, 
        cmdclass = {'test': Test, 'test_example':TestExample})

auth_name = "Pierre Cladé"
setup(author=auth_name,
  maintainer=auth_name,
  long_description = long_description.format(auth_name=auth_name), 
  **setup_parameters)




""" Initialize the constants from the NIDAQmx.h file

Read the NIDAQmx.h file and "execute" the #define command
"""

import re
from .DAQmxConfig import dot_h_file

include_file = open(dot_h_file,'r') #Open NIDAQmx.h file

# Try get the version of NIDAQmx
preamble = []
for line in include_file:
    if line.startswith('/*'):
        preamble.append(line)
    else:
        break

for copyright_line in preamble:
    if "Copyright" in copyright_line:
        DAQmx_copyright_year = max([int(elm) for elm in re.findall('\d\d\d\d', copyright_line)])
        break
else:
    DAQmx_copyright_year = 2003

# Regular expression to parse the #define line
# Parse line like : #define PI 3.141592
# The first group is the name of the constant
# The second group the value
define = re.compile(r'\#define\s+(\S+)\s+(".*"|\S+)')

# List containing all the name of the constant
constant_list = []

for line in include_file:
    m = define.match(line)
    if m:
       name = m.group(1)
       value = m.group(2)
       try:
           exec(name+'='+value)
       except NameError:
           pass
       except SyntaxError:
           pass
       else:
           constant_list.append(name)

include_file.close()
